# create a new view (mode 0) with an empty layout
#Ruby script for creating airbridge structure
#using my custom function in external file. As you can see it doesn't matter that
#put the reference to the cell before I defined what the cell is
require_relative 'SK_library'

app = RBA::Application.instance
mw = app.main_window

#  create a new layout 
mw.create_layout( 0 )
view = mw.current_view

#  create a new layer in that layout
layout = view.cellview( 0 ).layout 
linfo = RBA::LayerInfo.new ("Silicon")

silicon_layer = layout.insert_layer( linfo )

# set the database unit (shown as an example, the default is 0.001)
#if you are going to work in microns, need one order of magnitude smaller to prevent
#snapping to too rough a grid
layout.dbu = 0.1

chip_length=62000
strip_width=90
x_shift=7400 #length of bond pad + safety gap
#this would depend on the width of the strips cutting up the ground plane
bridge_gap=200
bridge_gap2=200
bond_shift=50 #to overlap bond with waveguide nicely
strip_length=(chip_length-x_shift*2-bridge_gap)/2+bond_shift
#total width of strip with insulation
gap=170
dimple_width=130
bridge_width=120

# create a cell
cell1 = layout.create_cell("TOP")

lead_cell = layout.create_cell("BOND_CPW")
# create another layer
metal_layer = layout.insert_layer(RBA::LayerInfo::new("Metal"))

# add a shape
cell1.shapes(metal_layer).insert(RBA::Box::new(0, 0, chip_length, chip_length))

cpw_cell = layout.create_cell("CPW")

waveguide(silicon_layer,cpw_cell,"(0,0; #{strip_length},0)",strip_width,gap)

cpw_bridge_cell = layout.create_cell("CPWBRDG")
waveguide(silicon_layer,cpw_bridge_cell,"(0,0; #{bridge_gap},0)",strip_width,gap)
cpw_bridge_ref=cpw_bridge_cell.ref
cpw_bridge_ref.trans=RBA::Trans::new(strip_length+x_shift-50,6000+10000*5)
cell1.insert(cpw_bridge_ref)


#t1 is a transformation constant. 
t1 = RBA::CplxTrans::new(1, 0, false, x_shift-bond_shift, 6000)
elec_array=RBA::CellInstArray::new(cpw_cell.cell_index,t1)
lead_cell.insert(elec_array)

#t2 is a transformation constant. 
t2 = RBA::CplxTrans::new(1, 0, false, x_shift, 6000)
bond_cell = layout.create_cell("BOND")
bond_poly=bond(silicon_layer,bond_cell,strip_width,bond_shift) #creates bond and returns polygon
bond_ref=RBA::CellInstArray::new(bond_cell.cell_index,t2)

lead_cell.insert(bond_ref)

#lead_ref=RBA::CellInstArray::new(lead_cell.cell_index,RBA::CplxTrans::new)
lead_ref=lead_cell.ref #my own overloaded method for this object
lead_ref.na=1
lead_ref.nb=6
lead_ref.a=RBA::Point::new(2000,0)
lead_ref.b=RBA::Point::new(0,10000)

lead_ref2=lead_ref.dup
t3=RBA::CplxTrans::new(1, 180, true, chip_length, 0)#mirroring before rotation to prevent shift
lead_ref2.transform(t3)

cell1.insert(lead_ref)
cell1.insert(lead_ref2)

#Speed demo- puts a lot of holes in the base layer
#hole_cell = layout.create_cell("HOLE")
#hole_cell.shapes(silicon_layer).insert(RBA::Box::new(0, 0, 50, 50))
#hole_ref=RBA::CellInstArray::new(hole_cell.ref)
#hole_ref.na=300
#hole_ref.nb=300
#hole_ref.a=RBA::Point::new(200,0)
#hole_ref.b=RBA::Point::new(0,200)
#cell1.insert(hole_ref)

#first layer marker
marker_width=100
marker_length=1000
offsetM=1200
marker_cell= marker(marker_width, marker_length, "MARKER",silicon_layer,20,layout)
marker_cell.shapes(silicon_layer).insert(RBA::Text::new("DRSK",RBA::Trans::new,100,1))

marker_ref=marker_cell.ref
marker_ref.na=2
marker_ref.nb=2
marker_ref.trans=RBA::Trans::new(0,offsetM,offsetM)
marker_ref.a=RBA::Point::new(0,chip_length-offsetM-marker_length -marker_width)
marker_ref.b=RBA::Point::new(chip_length-offsetM-marker_length -marker_width,0)
cell1.insert(marker_ref)

bridge_gap_cell = layout.create_cell("GAP")
bridge_gap_cell.shapes(silicon_layer).insert(RBA::Box::new(0, -gap/2, (gap-strip_width)/2, gap/2))
bridge_gap_cell.shapes(silicon_layer).insert(RBA::Box::new(bridge_gap, -gap/2, bridge_gap-(gap-strip_width)/2, gap/2))
gap_ref=lead_ref.dup
gap_ref.cell_index=bridge_gap_cell.cell_index
gap_ref.trans=RBA::Trans::new(strip_length+x_shift-50,6000)
gap_ref.nb=5
cell1.insert(gap_ref)

linfo = RBA::LayerInfo.new ("DIMPLES")
dimple_layer = layout.insert_layer( linfo )
dimple_ref=lead_ref.dup
dimp_cell = layout.create_cell("DIMPLE")
#Here the gap is being chosen for the dimples from the ground plane. Getting too close might cause a short
dimp_cell.shapes(dimple_layer).insert(RBA::Box::new(-100, -gap/2+20, 0, gap/2-20))
dimp_cell.shapes(dimple_layer).insert(RBA::Box::new(bridge_gap+100, -gap/2+20, bridge_gap, gap/2-20))
dimple_ref.cell_index=dimp_cell.cell_index
dimple_ref.trans=RBA::Trans::new(strip_length+x_shift-50,6000)
dimple_ref.nb=5

marker2X=offsetM+2500
marker_cell2= marker(marker_width, marker_length, "MARKER2",dimple_layer,20,layout)
marker_ref2=marker_cell2.ref
marker_ref2.na=2
marker_ref2.nb=2
marker_ref2.trans=RBA::Trans::new(0,marker2X,offsetM)
marker_ref2.a=RBA::Point::new(0,chip_length-offsetM-marker_length -marker_width)
marker_ref2.b=RBA::Point::new(chip_length-marker_length-marker2X-2500-marker_width,0)

linfo = RBA::LayerInfo.new ("BRIDGE")
bridge_layer = layout.insert_layer( linfo )
bridge_cell = layout.create_cell("BRIDGE")
#here we are going to put this 13um wide bridge with increasing shifts to check best location
#first we cover everything
chipP =  RBA::Polygon::new(RBA::Box::new(0,0,chip_length,chip_length))
bridgeB=RBA::Box::new(-100, -gap/2+20, bridge_gap+100, gap/2-20)
bridgeB.move(strip_length+x_shift-50,6000)
for yshift in 0..4
  chipP.insert_hole(bridgeB)
  bridgeB.move(0,10010)
end
bridge_cell.shapes(bridge_layer).insert(chipP)
bridge_ref=bridge_cell.ref
#bridge_ref.trans=RBA::Trans::new(strip_length+x_shift-50,6000)
cell1.insert(bridge_ref)
cell1.insert(marker_ref2)

#Some extra shapes added for guiding the cutting of the chip,
#visually inspecting the progress of aluminum acid etching and orientation
#borders
cell1.shapes(silicon_layer).insert(RBA::Box::new(-1000,0,0,chip_length))
cell1.shapes(silicon_layer).insert(RBA::Box::new(chip_length,0,chip_length+1000,chip_length))
cell1.shapes(silicon_layer).insert(RBA::Box::new(-1000,-1000,chip_length+1000,0))
cell1.shapes(silicon_layer).insert(RBA::Box::new(-1000,chip_length ,chip_length+1000,chip_length+1000))
#This side up triangle
triY=31000
cell1.shapes(silicon_layer).insert(RBA::Polygon::from_s("(#{chip_length/2},#{chip_length/2+triY};#{chip_length/2-5000},\
#{chip_length/2-5000+triY};#{chip_length/2+5000},#{chip_length/2-5000+triY})"))


#same thing for dimple layer
cell1.shapes(dimple_layer).insert(RBA::Box::new(-1000,0,0,chip_length))
cell1.shapes(dimple_layer).insert(RBA::Box::new(chip_length,0,chip_length+1000,chip_length))
cell1.shapes(dimple_layer).insert(RBA::Box::new(-1000,-1000,chip_length+1000,0))
cell1.shapes(dimple_layer).insert(RBA::Box::new(-1000,chip_length ,chip_length+1000,chip_length+1000))
#This side up
triY=31000
cell1.shapes(dimple_layer).insert(RBA::Polygon::from_s("(#{chip_length/2},#{chip_length/2+triY};#{chip_length/2-5000},\
#{chip_length/2-5000+triY};#{chip_length/2+5000},#{chip_length/2-5000+triY})"))


#processing all silicon and metal shapes so can be subtracted
#This is only needed for negative photoresist!
#xlayer=RBA::LayerInfo.new ("XOR")
#xor_layer = layout.insert_layer( xlayer)
#shapeP=RBA::ShapeProcessor::new
#silicon_shape_array=Array.new
#cell1.flatten(true)
#cell1.each_shape(silicon_layer) { |i|  silicon_shape_array<<(i) }
#siliconP= shapeP.merge_to_polygon(silicon_shape_array,0,true,true)
#metal_shapes=cell1.shapes(metal_layer)
#metal_shape_array=Array.new
#cell1.each_shape(metal_layer) { |i|  metal_shape_array<<(i) }
#outP = shapeP.boolean_to_polygon(silicon_shape_array,metal_shape_array, RBA::EdgeProcessor::ModeBNotA,true,false)
#regular boolean returns useless edge object
#cell_xor = layout.create_cell("RESULT")
#outP.each { |i|  cell_xor.shapes(xor_layer).insert(i) }
#cell1.insert(cell_xor.ref)

cell1.insert(dimple_ref)
cell1.flatten(true)

# select the top cell in the view, set up the view's layer list and
# fit the viewport to the extensions of our layout
view.select_cell(cell1.cell_index, 0)
layout.delete_layer(metal_layer)
view.add_missing_layers

#view.zoom_fit


